@@ -1,3 +1,28 @@
+# Copyright (c) 2007, Columbia Center For New Media Teaching And Learning (CCNMTL)
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#     * Redistributions of source code must retain the above copyright
+#       notice, this list of conditions and the following disclaimer.
+#     * Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
+#     * Neither the name of the CCNMTL nor the
+#       names of its contributors may be used to endorse or promote products
+#       derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY CCNMTL ``AS IS'' AND ANY
+# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
+# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 """ Test Suite for restclient
 
 Contributed by Christopher Hesse, modified by Anders Pearson
@@ -16,10 +41,15 @@ from restclient import *
 import threading, os
 import BaseHTTPServer
 import cgi
+import random
 
-port_num = int(os.environ.get('RESTCLIENT_TEST_PORT',11123))
+random.seed()
+port_num = int(os.environ.get('RESTCLIENT_TEST_PORT',random.randint(10000,11000)))
 hostname = "http://localhost:%d/" % port_num
-image = open('sample.jpg').read()
+image = open(os.path.join(os.getcwd(), 'sample.jpg')).read()
+
+# default headers
+headers = {'accept-encoding': 'compress, gzip'}
 
 def start_server(callback):
     class LoopbackHandler(BaseHTTPServer.BaseHTTPRequestHandler):
@@ -62,42 +92,49 @@ def start_server(callback):
         server_address = ('', port_num)
         httpd = server_class(server_address, handler_class)
         httpd.handle_request()
-
+        
     thread = threading.Thread(target=run)
     thread.setDaemon(True)
     thread.start()
     callback()
-
+    
 
 def servify(f):
     def test(*args, **kwargs):
         def run():
             f(*args, **kwargs)
+        global port_num, hostname
+        port_num += 1
+        hostname = "http://localhost:%d/" % port_num
         start_server(run)
+    test.__doc__ = f.__doc__
     return test
 
 
 @servify
 def test_get():
-    expected = "GET / HTTP/1.1\nHost: localhost:11123\r\ncontent-length: 0\r\n" + \
-    "content-type: application/x-www-form-urlencoded\r\naccept-encoding: compress, gzip\r\n" + \
-    "accept: */*\r\nuser-agent: Python-httplib2/$Rev: 133 $\r\n\n\n"
-
-    r = GET(hostname)
-    assert r.startswith('GET /')
-    assert r == expected
+    "Testing GET request"
     
+    expected = "GET / HTTP/1.1\nHost: localhost:%s\r\ncontent-length: 0\r\n"%(port_num) + \
+    "content-type: application/x-www-form-urlencoded\r\naccept-encoding: compress, gzip\r\n" + \
+    "accept: */*\r\nuser-agent: Python-httplib2/%s \r\n\n\n"%httplib2.__version__
+
+    r = GET(hostname, headers=headers)
+    assert r.startswith('GET /')
+    assert r.strip() == expected.strip()
 
 @servify
 def test_post():
+    "Testing POST request without body"
     expected = "POST\nvalue: store this\nDONE\n"
     r = POST(hostname, params={'value' : 'store this'}, accept=["text/plain","text/html"], async=False)
     assert r.startswith('POST /')
     assert "value=store+this" in r
     assert "accept: text/plain,text/html" in r
-
+    
 @servify
 def test_post_image():
+    "Testing POST with image without body"
     result = POST(hostname + "resize", files={'image' : {'file' : image, 'filename' : 'sample.jpg'}},
                   async=False)
     assert result.startswith('POST /resize')
@@ -105,6 +142,8 @@ def test_post_image():
 
 @servify
 def test_get_unicode():
+    "Testing GET with Unicode"
+    
     expected = u"GET\nfoo\u2012: \u2012\nDONE\n".encode('utf-8')
     r = GET(unicode(hostname + "foo/"),params={u'foo\u2012' : u'\u2012'},
             headers={u"foo\u2012" : u"foo\u2012"})
@@ -115,6 +154,8 @@ def test_get_unicode():
 
 @servify
 def test_post_unicode():
+    "Testing POST Unicode without body"
+    
     result = POST(unicode(hostname + "foo/"), 
                   params={u'foo\u2012' : u'\u2012'},
                   async=False)
@@ -122,9 +163,18 @@ def test_post_unicode():
     expected = "foo%E2%80%92=%E2%80%92" # urlencoded 
     assert expected in result
 
+@servify
+def test_post_with_body():
+    "Testing POST request with body"
+    body = '<test><string>Some text</string></test>'
+    r = POST(hostname, body=body, accept=["text/plain","text/html"], async=False )
+    assert r.startswith('POST /')
+    assert body in r
 
-if __name__ == "__main__":
+def main():
     import nose
     nose.main()
 
+if __name__ == "__main__":
+    main()
  
